@@ -3,7 +3,7 @@ use 5.008005;
 use strict;
 use warnings;
 
-our $VERSION = "1.08";
+our $VERSION = "1.12";
 
 sub ident_plain {
     'h2c';
@@ -34,7 +34,7 @@ Protocol::HTTP2 - HTTP/2 protocol implementation (RFC 7540)
 
 =head1 DESCRIPTION
 
-Protocol::HTTP2 is HTTP/2 protocol implementation (RFC 7540) with stateful
+Protocol::HTTP2 is HTTP/2 protocol implementation (L<RFC 7540|https://tools.ietf.org/html/rfc7540>) with stateful
 decoders/encoders of HTTP/2 frames. You may use this module to implement your
 own HTTP/2 client/server/intermediate on top of your favorite event loop over
 plain or tls socket (see examples).
@@ -74,7 +74,7 @@ never become production, but at least it works.
 
 =over
 
-=item - -- not implemeted
+=item - -- not implemented
 
 =item ± -- incomplete
 
@@ -101,8 +101,7 @@ L<Protocol::HTTP2::Stream> (stream operations) and L<Protocol::HTTP2::Upgrade>
 
 =head2 L<Protocol::HTTP2::HeaderCompression>
 
-Module implements HPACK - Header Compression for HTTP/2 (RFC 7541).
-L<https://tools.ietf.org/html/rfc7541>
+Module implements HPACK - Header Compression for HTTP/2 (L<RFC 7541|https://tools.ietf.org/html/rfc7541>).
 
 =head2 L<Protocol::HTTP2::Constants>
 
